import argparse
from html.parser import HTMLParser
import re
import subprocess
import urllib.request

def call(command):
  return subprocess.Popen(command, shell=True, executable='/bin/bash',
    stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()

class DirectoryHTMLParser(HTMLParser):
  def __init__(self):
    HTMLParser.__init__(self)
    self.link = None

  def handle_starttag(self, tag, attrs):
    if tag == 'a':
      for attr in attrs:
        if len(attr) > 1 and attr[0] == 'href':
          link = attr[1]
          m = re.search(r'(\d+)', link)
          if m is not None and (self.link is None or int(m.group(1)) >
              int(self.link)):
            self.link = m.group(1)

class FileHTMLParser(HTMLParser):
  def __init__(self):
    HTMLParser.__init__(self)
    self.link = None

  def handle_starttag(self, tag, attrs):
    if tag == 'a':
      for attr in attrs:
        if len(attr) > 1 and attr[0] == 'href':
          link = attr[1]
          m = re.search(r'nexus-\d+\.tar\.gz', link)
          if m is not None:
            self.link = m.group(0)

def main():
  parser = argparse.ArgumentParser(
    description='v1.0 Copyright (C) 2020 Spire Trading Inc.')
  parser.add_argument('--python', action='store_true',
    help='Only install Python modules.')
  parser.add_argument('-s', '--site', type=str, help='Site hosting the build.',
    default='https://builds.spiretrading.com')
  args = parser.parse_args()
  sub_directory = '/Linux-x86-64/Nexus'
  path = '%s%s' % (args.site, sub_directory)
  response = urllib.request.urlopen(path)
  parser = DirectoryHTMLParser()
  parser.feed(response.read().decode('utf-8'))
  if parser.link is None:
    print('Build not found.')
    return
  build_version = parser.link
  response = urllib.request.urlopen('%s/%s' % (path, build_version))
  parser = FileHTMLParser()
  parser.feed(response.read().decode('utf-8'))
  if parser.link is None:
    print('Archive not found.')
    return
  archive = parser.link
  response = urllib.request.urlopen('%s/%s/%s' % (path, build_version, archive))
  call('touch %s' % archive)
  output = open(archive, 'wb')
  output.write(response.read())
  output.close()
  python_path = call('python3 -m site --user-site')[0].strip().decode('utf-8')
  if args.python:
    call('tar -xzf %s %s' % (archive, 'Python'))
    call('mkdir -p %s' % python_path)
    call('rm -f %s/aspen.so' % python_path)
    call('rm -f %s/beam.so' % python_path)
    call('rm -f %s/nexus.so' % python_path)
    call('mv Python/aspen.so %s' % python_path)
    call('mv Python/beam.so %s' % python_path)
    call('mv Python/nexus.so %s' % python_path)
    call('rm -rf Python')
    call('rm %s' % archive)
  else:
    call('tar -xzf %s' % archive)
    call('mkdir -p %s' % python_path)
    call('rm -f %s/aspen.so' % python_path)
    call('rm -f %s/beam.so' % python_path)
    call('rm -f %s/nexus.so' % python_path)
    call('mv Python/aspen.so %s' % python_path)
    call('mv Python/beam.so %s' % python_path)
    call('mv Python/nexus.so %s' % python_path)
    call('rm %s' % archive)

if __name__ == '__main__':
  main()
